import { Component, Input } from '@angular/core';

import { DataSource } from '../../lib/data-source/data-source';

@Component({
  selector: 'ng2-smart-table-pager',
  styles: [require('./pager.scss')],
  template: `
  <nav *ngIf="shouldShow()" class="ng2-smart-pagination-nav">
    <ul class="ng2-smart-pagination pagination">
      <li class="ng2-smart-page-item page-item" [ngClass]="{disabled: getPage() == 1}">
        <a class="ng2-smart-page-link page-link" href="#" 
        (click)="getPage() == 1 ? false : paginate(1)" aria-label="First">
          <span aria-hidden="true">&laquo;</span>
          <span class="sr-only">First</span>
        </a>
      </li>
      <li class="ng2-smart-page-item page-item" 
      [ngClass]="{active: getPage() == page}" *ngFor="let page of getPages()">
        <span class="ng2-smart-page-link page-link" 
        *ngIf="getPage() == page">{{ page }} <span class="sr-only">(current)</span></span>
        <a class="ng2-smart-page-link page-link" href="#" 
        (click)="paginate(page)" *ngIf="getPage() != page">{{ page }}</a>
      </li>

      <li class="ng2-smart-page-item page-item" 
      [ngClass]="{disabled: getPage() == getLast()}">
        <a class="ng2-smart-page-link page-link" href="#" 
        (click)="getPage() == getLast() ? false : paginate(getLast())" aria-label="Last">
          <span aria-hidden="true">&raquo;</span>
          <span class="sr-only">Last</span>
        </a>
      </li>
    </ul>
  </nav>
  `
})
export class PagerComponent {

  @Input() perPage: number;
  @Input() source: DataSource;

  protected pages: Array<any>;
  protected page: number;
  protected count: number = 0;

  ngOnInit(): void {
    this.source.onChanged().subscribe((changes) => {
      this.page = this.source.getPaging().page;
      this.count = this.source.count();

      if (this.isPageOutOfBounce()) {
        this.source.setPage(--this.page);
      }

      this.processPageChange(changes);
      this.initPages();
    });
  }

  /**
   * We change the page here depending on the action performed against data source
   * if a new element was added to the end of the table - then change the page to the last
   * if a new element was added to the beginning of the table - then to the first page
   * @param changes
   */
  processPageChange(changes): void {
    if (changes['action'] === 'prepend') {
      this.source.setPage(1);
    }
    if (changes['action'] === 'append') {
      this.source.setPage(this.getLast());
    }
  }

  shouldShow(): boolean {
    return this.source.count() > this.perPage;
  }

  paginate(page: number): boolean {
    this.source.setPage(page);
    this.page = page;
    return false;
  }

  getPage(): number {
    return this.page;
  }

  getPages(): Array<any> {
    return this.pages;
  }

  getLast(): number {
    return Math.ceil(this.count / this.perPage);
  }

  protected isPageOutOfBounce(): boolean {
    return (this.page * this.perPage) >= (this.count + this.perPage) && this.page > 1;
  }

  protected initPages() {
    let pagesCount = this.getLast();
    let showPagesCount = 4;
    showPagesCount = pagesCount < showPagesCount ? pagesCount : showPagesCount;
    this.pages = [];

    if (this.shouldShow()) {

      let middleOne = Math.ceil(showPagesCount / 2);
      middleOne = this.page >= middleOne ? this.page : middleOne;

      let lastOne = middleOne + Math.floor(showPagesCount / 2);
      lastOne = lastOne >= pagesCount ? pagesCount : lastOne;

      let firstOne = lastOne - showPagesCount + 1;

      for (let i = firstOne; i <= lastOne; i++) {
        this.pages.push(i);
      }
    }
  }
}
